@@ -578,7 +578,9 @@ class DictStack(list, collections.abc.Mapping):
     3
     >>> set(stack.keys()) == set(['a', 'b', 'c'])
     True
-    >>> dict(**stack) == dict(a=3, c=2, b=2)
+    >>> set(stack.items()) == set([('a', 3), ('b', 2), ('c', 2)])
+    True
+    >>> dict(**stack) == dict(stack) == dict(a=3, c=2, b=2)
     True
     >>> d = stack.pop()
     >>> stack['a']
@@ -587,10 +589,13 @@ class DictStack(list, collections.abc.Mapping):
     >>> stack['a']
     1
     >>> stack.get('b', None)
+    >>> 'c' in stack
+    True
     """
 
-    def keys(self):
-        return list(set(itertools.chain.from_iterable(c.keys() for c in self)))
+    def __iter__(self):
+        dicts = list.__iter__(self)
+        return iter(set(itertools.chain.from_iterable(c.keys() for c in dicts)))
 
     def __getitem__(self, key):
         for scope in reversed(self):
@@ -600,6 +605,9 @@ class DictStack(list, collections.abc.Mapping):
 
     push = list.append
 
+    def __contains__(self, other):
+        return collections.abc.Mapping.__contains__(self, other)
+
 
 class BijectiveMap(dict):
     """
